﻿-- Create Ace3 instance
SoDEC = LibStub("AceAddon-3.0"):NewAddon("SoDEndlessChat", "AceConsole-3.0", "AceHook-3.0")

-- Global varables
local _G = _G

-- varables
local defaultMaxLetters = 255
local defaultMaxBytes = 256

------------------------------------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------------------------------------
-------------------------------------------------------------- Initialisation Functions --------------------------------------------------------------------
function SoDEC:OnInitialize()
end
------------------------------------------------------------------------------------------------------------------------------------------------------------
function SoDEC:OnEnable()
	self:SecureHook("ChatEdit_UpdateHeader", "ChatEdit_Update")
	self:RawHook("SendChatMessage", true)
	self:RawHook("BNSendWhisper", true)
	self:RawHook("BNSendConversationMessage", true)
	
	local editBox = _G.DEFAULT_CHAT_FRAME.editBox
	defaultMaxLetters = editBox:GetMaxLetters()
	defaultMaxBytes = editBox:GetMaxBytes()	
end
------------------------------------------------------------------------------------------------------------------------------------------------------------
function SoDEC:OnDisable()
end
------------------------------------------------------------------------------------------------------------------------------------------------------------
function SoDEC:OnProfileChanged()
end
------------------------------------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------- Call Functions -------------------------------------------------------------------------
function SoDEC:ChatEdit_Update(self)
	local activeWindow = _G.ChatEdit_GetActiveWindow()
	if not activeWindow then return end
	
	local chattype = self:GetAttribute("chatType")
	if (chattype == "GUILD" or chattype == "OFFICER" or chattype == "PARTY" or chattype == "RAID" or chattype == "SAY" or chattype == "EMOTE" or chattype == "INSTANCE_CHAT" or chattype == "CHANNEL" or chattype == "WHISPER" or chattype == "BN_CONVERSATION" or chattype == "BN_WHISPER") then
		self:SetMaxLetters(0)
		self:SetMaxBytes(0)
	else
		self:SetMaxLetters(defaultMaxLetters)
		self:SetMaxBytes(defaultMaxBytes)
	end
end
------------------------------------------------------------------------------------------------------------------------------------------------------------
function SoDEC:BNSendConversationMessage(...)
	local BNETID, msg = ...
	if string.len(msg) > defaultMaxLetters then
		local chunks = SoDEC:SplitIntoChunks(msg)
		for i=1, #chunks do 
			_G.BNSendConversationMessage(BNETID, chunks[i])
		end
		return
	end
	SoDEC.hooks.BNSendConversationMessage(...)
end
------------------------------------------------------------------------------------------------------------------------------------------------------------
function SoDEC:BNSendWhisper(...)
	local BNETID, msg = ...
	if string.len(msg) > defaultMaxLetters then
		local chunks = SoDEC:SplitIntoChunks(msg)
		for i=1, #chunks do 
			_G.BNSendWhisper(BNETID, chunks[i])
		end
		return
	end
	SoDEC.hooks.BNSendWhisper(...)
end
------------------------------------------------------------------------------------------------------------------------------------------------------------
function SoDEC:SendChatMessage(...)
	local msg, chatType, language, channel = ...
	if string.len(msg) > defaultMaxLetters then
		local chunks = SoDEC:SplitIntoChunks(msg)
		for i=1, #chunks do 
			_G.ChatThrottleLib:SendChatMessage("ALERT", "SoDEC", chunks[i], chatType, language, channel)
		end
		return
	end
	SoDEC.hooks.SendChatMessage(...)
end
------------------------------------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------- TOOL Functions -------------------------------------------------------------------------
function SoDEC:SplitIntoChunks(longMsg)
	local splitMessageLinks, chunks, words, temp = {}, {}, {}, ""
	
	-- Parse out links
	longMsg, results = string.gsub(longMsg, "(|H[^|]+|h[^|]+|h)", function(theLink)
				table.insert(splitMessageLinks, theLink)
				return "\001\002"..SoDEC:paddString(#splitMessageLinks, "0", string.len(theLink)-4).."\003\004"
	end)
	
	-- Breakdown Words
	for v in string.gmatch(longMsg, "[^ ]+") do
		if string.len(v) > defaultMaxLetters then
			local shortPart, remainingPart = nil, v
			local i = 1
			while (remainingPart and string.len(remainingPart) > 0) do
				shortPart, remainingPart = SoDEC:GetShorterString(remainingPart)
				if shortPart and shortPart ~= "" then table.insert(words, shortPart) end
				if i > 10 then break end
				i = i + 1
			end
		else table.insert(words, v) end
	end
	
	-- Recombine
	for i=1, #words + 1 do 
		if words[i] and (string.len(temp) + string.len(words[i])) <= (defaultMaxLetters - 1) then
			if string.len(temp) > 0 then temp = temp.." "..words[i]
			else temp = words[i] end
		else
			temp = string.gsub(temp, "\001\002%d+\003\004", function(link)
						local index = tonumber(string.match(link, "(%d+)"))
						return splitMessageLinks[index] or link
			end)
			table.insert(chunks, temp)
			if words[i] then temp = words[i] end
		end
	end
	return chunks
end
------------------------------------------------------------------------------------------------------------------------------------------------------------
-- UTF8 for Localization
function SoDEC:GetShorterString(longMsg)
	local shortText = string.utf8sub(longMsg, 1, defaultMaxLetters - 1)
	--local remainingPart = string.utf8sub(longMsg, defaultMaxLetters, string.len(longMsg))
	local remainingPart = string.utf8sub(longMsg, string.len(shortText) + 1, string.len(longMsg))
	return shortText, remainingPart
end
------------------------------------------------------------------------------------------------------------------------------------------------------------
function SoDEC:paddString(str, paddingChar, minLength, paddRight)
    str = tostring(str or "")
    paddingChar = tostring(paddingChar or " ")
    minLength = tonumber(minLength or 0)
    while string.len(str) < minLength do
        if paddRight then str = str..paddingChar
        else str = paddingChar..str end
    end
    return str
end